import numpy as np
import matplotlib.pyplot as plt
from scipy.stats import gaussian_kde
import matplotlib
import matplotlib.pyplot as plt
matplotlib.rcParams['mathtext.fontset'] = 'cm'
matplotlib.rcParams['font.family'] = 'STIXGeneral'

# 读取数据
path = '/Users/ivcylc_lca/Desktop/123/mos_fma_audioset.lst'
data = []
with open(path, 'r') as f:
    for line in f:
        data.append(float(line.strip()))

dir = '/Users/ivcylc_lca/Desktop/test.lst'
with open(dir, 'r') as file:
    data_ = [line.strip() for line in file]
data2 = []
for x in data_:
    try:
        data2.append(float(x))
    except:
        pass
data_new = data2

# 计算均值和标准差
miu = np.mean(data)
sigma = np.std(data)
miu_new = np.mean(data_new)
sigma_new = np.std(data_new)

# 自定义颜色
colors = [
    (158/255, 1/255, 66/255),   # 深色
    (213/255, 62/255, 79/255),
    (254/255, 224/255, 139/255),  # 浅色
    (244/255, 109/255, 67/255),
    (253/255, 174/255, 97/255),
]
blue_colors = [
    (198/255, 219/255, 239/255),  # 浅蓝色
    (158/255, 202/255, 225/255),  # 蓝色
    (107/255, 174/255, 216/255),  # 更深一点蓝色
    (33/255, 113/255, 181/255),   # 很深的蓝色
]

# 创建主图
fig, ax1 = plt.subplots(figsize=(8, 6))  # 调整图的大小为正方形

# 第一个数据集的密度估计
kde = gaussian_kde(data)
x = np.linspace(min(data), max(data), 1000)
y = kde(x)

# 归一化第一个数据集的密度
y_max = max(y)
a1 = 0.4
y = y * a1  # 将高度限制在Y轴总高度的一半

# 绘制第一个数据集的密度曲线
ax1.plot(x, y, color="red", linewidth=2)

# 填充第一个数据集的颜色
delta = 0.01
ax1.fill_between(x, y, where=(x >= miu - sigma - delta) & (x <= miu + sigma), color=colors[3], alpha=1, label='VQ-MOS: 3')
ax1.fill_between(x, y, where=(x >= miu - 2*sigma - delta) & (x <= miu - sigma), color=colors[4], alpha=1, label='VQ-MOS: 2')
ax1.fill_between(x, y, where=(x < miu - 2*sigma), color=colors[2], alpha=1, label='VQ-MOS: 1')
ax1.fill_between(x, y, where=(x >= miu + sigma - delta) & (x <= miu + 2*sigma + delta), color=colors[1], alpha=1, label='VQ-MOS: 4')
ax1.fill_between(x, y, where=(x >= miu + 2*sigma - delta), color=colors[0], alpha=1, label='VQ-MOS: 5')

# 创建共享X轴的第二个数据集的子图
ax2 = ax1.twiny()

# 第二个数据集的密度估计
kde_new = gaussian_kde(data_new)
x_new = np.linspace(min(data_new), max(data_new), 1000)
y_new = kde_new(x_new)

# 反转第二个数据集的Y值
y_new *= 0.17
y_new = 1 - y_new

# 绘制第二个数据集的密度曲线
ax2.plot(x_new, y_new, color="blue", linewidth=2)

# 填充第二个数据集的颜色在曲线之上
ax2.fill_between(x_new, 1, y_new, where=(x_new < 0), color=blue_colors[0], label='Error captioning')
ax2.fill_between(x_new, 1, y_new, where=(x_new >= 0) & (x_new < 0.1), color=blue_colors[1], label='Low Similarity')
ax2.fill_between(x_new, 1, y_new, where=(x_new >= 0.1) & (x_new < 0.3), color=blue_colors[2], label='Medium Similarity')
ax2.fill_between(x_new, 1, y_new, where=(x_new >= 0.3), color=blue_colors[3], label='Highly correlated')

# 设置双X轴的范围
ax1.set_xlim(2.85, 4.6)
ax2.set_xlim(-0.20, 0.8)
ax1.set_ylim(0, 1)
ax1.set_xlabel('Pesudo-MOS', fontsize=16, color='red')
ax2.set_xlabel('CLAP Similarity', fontsize=16, color='blue')

# 设置刻度线和标签的颜色
ax1.tick_params(axis='x', labelsize=16, colors='red', direction='out')
ax1.tick_params(axis='y', labelsize=16, colors='red', direction='out')
ax1.tick_params(axis='both', which='both', colors='red')

ax2.tick_params(axis='x', labelsize=16, colors='blue', direction='out')
ax2.tick_params(axis='y', labelsize=16, colors='blue', direction='out')
ax2.tick_params(axis='both', which='both', colors='blue')

# 确保spines颜色被正确设置
ax1.spines['bottom'].set_color('red')
ax1.spines['left'].set_color('red')
ax1.spines['top'].set_color('none')  # 移除顶部黑线
ax1.spines['right'].set_color('none')  # 移除右侧黑线
ax1.spines['bottom'].set_linewidth(2)
ax1.spines['left'].set_linewidth(2)

ax2.spines['top'].set_color('blue')
ax2.spines['right'].set_color('none')  # 移除右侧黑线
ax2.spines['bottom'].set_color('none')  # 移除底部黑线
ax2.spines['left'].set_color('none')  # 移除左侧黑线
ax2.spines['top'].set_linewidth(2)

# 创建共享Y轴的第三个数据集的子图
ax3 = ax1.twinx()

# 设置右边坐标轴的范围
ax3.set_ylim(1, 0)

# 设置右边坐标轴的颜色和宽度
ax3.spines['left'].set_color('none')
ax3.spines['bottom'].set_color('none')
ax3.spines['top'].set_color('none')
ax3.spines['right'].set_color('blue')
ax3.spines['right'].set_linewidth(2)

# 调整右边坐标轴的刻度标签颜色和大小
ax3.tick_params(axis='y', labelsize=16, colors='blue')
ax3.tick_params(axis='both', which='both', colors='blue')

# 确保子图的布局紧凑，并向右扩展一点
fig.subplots_adjust(left=0.05, right=0.95, top=0.9, bottom=0.1)

# 显示图表
plt.savefig('fengmian.pdf')
plt.show()
