import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt

# 读取数据
path = '/Users/ivcylc_lca/Desktop/123/mos_fma_audioset.lst'
data = []
with open(path, 'r') as f:
    for line in f:
        data.append(float(line.strip()))
ori = data

# 计算均值和标准差
miu = np.mean(data)
sigma = np.std(data)

# 设置lambda值
lambda_val = 7

# 计算加权后的分布
weighted_data = np.exp(lambda_val * np.array(data))
weighted_data /= np.sum(weighted_data)
weighted_data *= len(weighted_data)

# 对每个数据应用权重并随机选择是否保留
filtered_data = [data[i] for i in range(len(data)) if np.random.rand() < weighted_data[i]]

# 自定义颜色
colors = [
    (158/255, 1/255, 66/255),   # 深色
    (213/255, 62/255, 79/255),
    (254/255, 224/255, 139/255),  # 浅色
    (244/255, 109/255, 67/255),
    (253/255, 174/255, 97/255),
]

# 绘制剩余数据的密度曲线
plt.figure(figsize=(10, 6))  # 调整图的大小
sns.kdeplot(filtered_data, color="black", linewidth=2)

# 计算虚线的最大高度
kde = sns.kdeplot(filtered_data, color="black", linewidth=2).get_lines()[0].get_data()
kde_y = kde[1]
max_height = np.interp([miu+sigma, miu+2*sigma, miu-sigma, miu-2*sigma], kde[0], kde_y)

# 在特定的位置画虚线
plt.axvline(miu + sigma, color='black', linewidth=2, ymax=max_height[0]/plt.ylim()[1])
plt.axvline(miu + 2*sigma, color='black', linewidth=2, ymax=max_height[1]/plt.ylim()[1])
plt.axvline(miu - sigma, color='black', linewidth=2, ymax=max_height[2]/plt.ylim()[1])
plt.axvline(miu - 2*sigma, color='black', linewidth=2, ymax=max_height[3]/plt.ylim()[1])

# 填充颜色
x = kde[0]
y = kde[1]
delta = 0.01
plt.fill_between(x, y, where=(x >= miu - sigma -delta) & (x <= miu + sigma ), color=colors[3], alpha=1, label='VQ-MOS: 3')
plt.fill_between(x, y, where=(x >= miu - 2*sigma -delta) & (x <= miu - sigma ), color=colors[4], alpha=1, label='VQ-MOS: 2')
plt.fill_between(x, y, where=(x < miu - 2 *sigma), color=colors[2], alpha=1, label='VQ-MOS: 1')
plt.fill_between(x, y, where=(x >= miu + sigma-delta) & (x <= miu + 2*sigma+delta), color=colors[1], alpha=1, label='VQ-MOS: 4')
plt.fill_between(x, y, where=(x >= miu + 2* sigma-delta) , color=colors[0], alpha=1, label='VQ-MOS: 5')

# 去掉坐标轴
plt.axis('off')

# 在底部画一条从2.7到4.7的黑线
plt.xlim(2.85, 4.6)
plt.hlines(y=0, xmin=2.85, xmax=4.6, color='black', linewidth=5)

# 显示图表
plt.savefig('fengmian.png')
plt.show()
