import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import seaborn as sns
from matplotlib.patches import Polygon
import numpy as np
from matplotlib.patches import Rectangle
from matplotlib.lines import Line2D
from matplotlib.patches import ConnectionPatch
import matplotlib
import matplotlib.pyplot as plt
matplotlib.rcParams['mathtext.fontset'] = 'cm'
matplotlib.rcParams['font.family'] = 'STIXGeneral'

def add_triangle(ax, bar_center, bar_height, size=0.1):
    # size 控制三角形的大小
    # 定义三角形顶点的位置
    triangle = Polygon([(bar_center, bar_height + size*2.7),  # 顶点
                        (bar_center - size * 2 / 6, bar_height + size / 2),  # 左下角
                        (bar_center + size * 2 / 6, bar_height + size / 2)],  # 右下角
                       facecolor='purple',  # 三角形内部颜色
                       edgecolor='purple',  # 三角形边缘颜色
                       linewidth=1)  # 边缘线的宽度
    ax.add_patch(triangle)
    return triangle
# 测试数据和设置
categories = ['FAD', 'KL', 'IS', 'pMOS']
# Unet 模型数据
scores_unet_low = [6.568, 3.912, 2.119, 3.835]
scores_unet_medium = [6.035, 3.638, 1.999, 3.857]
scores_unet_high = [5.689, 3.611, 1.899, 3.917]

# Mdt 模型数据
scores_mdt_low = [9.228, 4.318, 2.773, 3.473]
scores_mdt_medium = [8.500, 4.100, 2.300, 3.843]  # 假设值
scores_mdt_high = [5.602, 3.357, 2.154, 4.039]
scores_mdt_ori = [5.757, 3.837, 2.508, 3.796]

# 主图设置
fig, ax = plt.subplots(figsize=(8, 6))
width = 0.08  # 柱子宽度
gap = 0.02    # 柱子之间的缝隙

# 创建柱状图位置
bars = np.arange(len(categories))*0.8  # 基本位置
bars_unet_low = bars - 2 * width - gap
bars_unet_medium = bars - width
bars_unet_high = bars + gap
bars_mdt_low = bars + width + 2 * gap
bars_mdt_medium = bars + 2 * width + 3 * gap
bars_mdt_high = bars + 3 * width + 4 * gap
bars_mdt_ori = bars + 4 * width + 5 * gap

# 指定 RGB 颜色
# unet_colors = [(171/255,217/255,233/255), (116/255,173/255,209/255), (69/255,117/255,180/255)]  # 浅蓝、中蓝、深蓝
# mdt_colors = [(253/255,174/255,97/255), (244/255,109/255,67/255), (215/255,48/255,39/255)]     # 浅红、中红、深红
unet_colors = [(199/255,234/255,229/255), (90/255,180/255,172/255), (1/255,102/255,94/255)]  # 浅蓝、中蓝、深蓝
mdt_colors = [(246/255,232/255,195/255), (216/255,179/255,101/255), (140/255,81/255,10/255)]     # 浅红、中红、深红
# 绘制主图柱状图
mdt_ori_color = (178/255,24/255,43/255)
ax.bar(bars_unet_low, scores_unet_low, width, label='U-Net Low', color=unet_colors[0])
ax.bar(bars_unet_medium, scores_unet_medium, width, label='U-Net Medium', color=unet_colors[1])
ax.bar(bars_unet_high, scores_unet_high, width, label='U-Net High', color=unet_colors[2])
ax.bar(bars_mdt_low, scores_mdt_low, width, label='MDT Low', color=mdt_colors[0])
ax.bar(bars_mdt_medium, scores_mdt_medium, width, label='MDT Medium', color=mdt_colors[1])
ax.bar(bars_mdt_high, scores_mdt_high, width, label='MDT High', color=mdt_colors[2])
ax.bar(bars_mdt_ori, scores_mdt_ori, width, label='MDT Without quality prefix', color=mdt_ori_color)

# 主图细节
# ax.set_xlabel('Metrics')
# ax.set_ylabel('Scores')
# ax.set_title('Performance Comparison of Unet and Mdt Models Across Settings')
ax.set_xticks(bars + width, fontsize=16)
ax.set_xticklabels(categories, fontsize=16)
ax.set_ylim([1.5, 9.5])
ax.legend(fontsize=12)

x_pos = 0.7  # x 坐标
y_pos = 4.5  # y 坐标
inset_fig_width = 0.95  # 嵌入图的宽度百分比
inset_fig_height = 2 * 0.95  # 嵌入图的高度百分比
ax_ins = inset_axes(ax, width=str(inset_fig_width*100) + "%", height=str(inset_fig_height*100) + "%", bbox_to_anchor=(x_pos, y_pos, inset_fig_width, inset_fig_height), bbox_transform=ax.transData, loc='lower left')

# 为嵌入图中的柱状图定义宽度
inset_width = 0.2  # 嵌入图中柱状图的宽度，你可以根据需要进一步调整
mos_index = categories.index('pMOS')  # 获取 MOS 在列表中的索引
mos_bars = np.arange(1)  # 只有一个数据点

ax_ins.bar(mos_bars - inset_width - gap, [scores_unet_low[mos_index]], inset_width, label='Unet Low', color=unet_colors[0])
ax_ins.bar(mos_bars, [scores_unet_medium[mos_index]], inset_width, label='Unet Medium', color=unet_colors[1])
ax_ins.bar(mos_bars + inset_width + gap, [scores_unet_high[mos_index]], inset_width, label='Unet High', color=unet_colors[2])

ax_ins.bar(mos_bars + 2 * inset_width + 2 * gap, [scores_mdt_low[mos_index]], inset_width, label='Mdt Low', color=mdt_colors[0])
ax_ins.bar(mos_bars + 3 * inset_width + 3 * gap, [scores_mdt_medium[mos_index]], inset_width, label='Mdt Medium', color=mdt_colors[1])
ax_ins.bar(mos_bars + 4 * inset_width + 4 * gap, [scores_mdt_high[mos_index]], inset_width, label='Mdt High', color=mdt_colors[2])
ax_ins.bar(mos_bars + 5 * inset_width + 5 * gap, [scores_mdt_ori[mos_index]], inset_width, label='Mdt Ori', color=mdt_ori_color)

# 设置嵌入图的细节
ax_ins.set_xticks(mos_bars + 2 * inset_width)  # 设置 x 轴刻度在中间的位置
ax_ins.set_xticklabels(['pMOS'])
ax_ins.set_ylim([3.40, 4.15])  # 设置 y 轴范围为 3 到 5
ax_ins.axhline(y=3.55, color='blue', linestyle='--', linewidth=3, label='Threshold')
ax_ins.axhline(y=3.902, color='red', linestyle='--', linewidth=3, label='MTT GT')
ax_ins.axhline(y=4.05, color='blue', linestyle='--', linewidth=3, label='Threshold')


# 在主图和嵌入图之间添加连接线
# 计算主图中'MOS'数据的位置
main_mos_position = bars + width + gap * 3  # 你可能需要调整这个以匹配你的布局
main_mos_height = scores_mdt_high[mos_index]  # 使用 Mdt High 的 MOS 分数作为高度

# 创建连接线
# 从主图到嵌入图
xyA = (main_mos_position[mos_index]-0.15, 4.5)  # 主图的点位置
xyB = (mos_bars + 4 * width + 4 * gap, scores_mdt_high[mos_index])  # 嵌入图的点位置
coordsA = "data"
coordsB = "data"
con = ConnectionPatch(xyA=xyA, xyB=xyB, coordsA=coordsA, coordsB=coordsB,
                      axesA=ax, axesB=ax_ins, arrowstyle="-|>", shrinkB=8, color="black")
ax.add_artist(con)

# 可以添加更多连接线，复制上面的代码并修改为连接其他数据点

# 计算 MOS 柱子的位置和宽度
mos_index = categories.index('pMOS')  # 获取 MOS 在列表中的索引
mos_x = bars[mos_index]-0.25  # 获取 MOS 的 x 位置
rect_width = width + 37 * gap  # 计算矩形的宽度

# 创建一个矩形，参数是 (x, y, width, height)
highlight = Rectangle((mos_x - width/2, 3.2), rect_width, 4.5 - 3.2, linewidth=1, edgecolor='black', facecolor='none')
ax.add_patch(highlight)



bar_center = bars_mdt_high[categories.index('pMOS')]
bar_height = scores_mdt_high[categories.index('pMOS')]
add_triangle(ax, bar_center, bar_height)

bar_center = bars_mdt_high[categories.index('FAD')]
bar_height = scores_mdt_high[categories.index('FAD')]
add_triangle(ax, bar_center, bar_height)

bar_center = bars_mdt_high[categories.index('KL')]
bar_height = scores_mdt_high[categories.index('KL')]
add_triangle(ax, bar_center, bar_height)

bar_center = bars_mdt_low[categories.index('IS')]
bar_height = scores_mdt_low[categories.index('IS')]
add_triangle(ax, bar_center, bar_height)

bar_center = bars_mdt_low[categories.index('IS')]
bar_height = scores_mdt_low[categories.index('IS')]
add_triangle(ax_ins, bar_center, bar_height)


legend_triangle = Line2D([0], [0], color='purple', marker='^', linestyle='None',
                         markersize=10, markerfacecolor='purple', label='Best Score')
existing_legend = ax.get_legend_handles_labels()
current_handles, current_labels = existing_legend[0], existing_legend[1]

# 添加新的图例条目
current_handles.append(legend_triangle)
current_labels.append('Best Score')

# 使用更新后的handles和labels设置图例
ax.legend(handles=current_handles, labels=current_labels)

# 保存和显示图形
plt.savefig('model_performance_comparison.pdf')
plt.show()
